<?php
/**
 * Class created by Paulo Coutinho - Brasil - www.prsolucoes.com
 * This will help to organize the extensions and automate the extension install process
 * @date 05-feb-2010
 */
class ExtensionCommand extends CConsoleCommand
{
    private $extensionsUrl;
    private $command;
    private $extensionName;
    private $showDebug;
    private $extensionContentPage;
    private $tempDir;
    private $extensionFormat;
    private $args;
    private $downloadMode;
    
	public function getHelp()
	{
		return <<<EOD
USAGE
  extension install <extension-name>

DESCRIPTION
  This command will install an extension hosted on Yii (http://www.yiiframework.com/extensions/)

EXAMPLES
 * Install the 'swfobject'' extension
        extension install swfobject        
EOD;
	}
    
    /**
     * Run the main command method
     * @param array     
     */
    public function run($args)
    {
        // store the args
        $this->args = $args;
        
        // initialize the properties
        $this->initialize();
        
        // verify the command name
        if (count($args) >= 2 && trim($args[0]) == 'install' && trim($args[1]) != '')
        {
            // prepare arguments
            $this->command       = trim($args[0]);
            $this->extensionName = trim($args[1]);
            
            // verify if the extension exists
            $this->extensionContentPage = $this->getUrlContent($this->extensionsUrl . $this->extensionName);
            
            $extensionExists = $this->verifyIfExists();
            
            if ($extensionExists == false)
            {
                $this->showFatalError('The extension not exists, please verify the extension name and try again.');
            }
                        
            // get the last extension file url
            $this->getExtensionFile();
            
            // execute the init file
            $this->executeInitFile();
            
            $this->showDebug('Extension installed sucessfully.');
                                     
        } else {
            echo($this->getHelp());
			return;
        }
    }
    
    /**
     * Initialize the command
     */
    private function initialize()
    {   
        set_time_limit(0);
        
        $this->showDebug       = true;
        $this->downloadMode    = 'curl'; //possible (curl, native)
        
        $this->extensionsUrl   = 'http://www.yiiframework.com/extension/';
        $this->command         = '';                
        $this->extensionName   = '';
        $this->tempDir         = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'extension' . DIRECTORY_SEPARATOR . 'tmp' . DIRECTORY_SEPARATOR;
        $this->extensionFormat = 'zip'; //possible(only zip)
    }
    
    /**
     * Verify if an extension really exists
     * @return boolean
     */
    private function verifyIfExists()
    {
        // show debug
        $this->showDebug('Verifying if the extension exists...');
        
        // initialize
        $result = false;
        
        // check error messages
        if (strpos($this->extensionContentPage, '<h1>Page Not Found</h1>') === false && trim($this->extensionContentPage) != '')
        {
            $result = true;
        }
        
        // show debug
        if ($result == true)
        {
            $this->showDebug('Yes, the extension exists.');
        }
        else
        {
            $this->showDebug('No, the extension not exists.');
        }
        
        return $result;
    }
    
    /**
     * Get url content
     * @param string 
     * @return string
     */
    private function getUrlContent($url)
    {   
        if ($this->downloadMode == 'curl')
        {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $pageContent = curl_exec($ch);
            curl_close ($ch);
        }
        else if ($this->downloadMode == 'native')
        {
            $pageContent = file_get_contents($url);
        }
        else
        {
            $this->showFatalError('Invalid download mode (possible: curl, native).');
        }
                
        return $pageContent;
    }
    
    /**
     * Show debug message if enabled
     * @param string 
     * @param boolean
     */
    public function showDebug($message, $autoNewLine = true)
    {
        if ($this->showDebug == true)
        {
            echo('DEBUG: ' . $message . ($autoNewLine == true ? "\n" : '') );
        }
    }
    
    /**
     * Show fatal error message and exit
     * @param string 
     * @param boolean
     * @param boolean
     */
    private function showFatalError($message, $autoExit = true, $autoNewLine = true)
    {
        echo('ERROR: ' . $message . ($autoNewLine == true ? "\n" : '') );
        
        if ($autoExit == true)
        {
            exit;
        }
    }
    
    /**
     * Create a new directory
     * @param string 
     */
    private function makeDir($path)
    {
        if (!is_dir($path))
        {
            mkdir($path);
        }
    }
    
    /**
     * Get the extension file
     */
    private function getExtensionFile()
    {
        // create the temporary dirs
        $this->createTemporaryDirs();
        
        // show debug
        $this->showDebug('Getting the extension file...');
        
        // download the file extension
        $fileName = $this->tempDir . $this->extensionName . DIRECTORY_SEPARATOR . $this->extensionName . '_last.' . $this->extensionFormat;
        
        $fileContent = $this->getUrlContent($this->extensionsUrl . $this->extensionName . '/' . 'files' . '/' . $this->extensionName . '_last.' . $this->extensionFormat);
        file_put_contents($fileName, $fileContent);
        
        // show debug
        $this->showDebug('Download complete.');
        
        // unpack the file
        $this->unpackFile($fileName);
    }
    
    /**
     * Create the temporary directories
     */
    private function createTemporaryDirs()
    {
        // show debug
        $this->showDebug('Creating the temporary directories...');
        
        // create the temporary dir
        $this->makeDir($this->tempDir);
        
        // create the temporary extension dir
        $this->makeDir($this->tempDir . $this->extensionName . DIRECTORY_SEPARATOR);
        
    }
    
    /**
     * Unpack a file
     * @param string
     * @param string
     */
    private function unpackFile($file, $path = '')
    {
        // show debug
        $this->showDebug('Unpacking...');
        
        // verify the path
        if (trim($path) == '')
        {
            $path = dirname($file) . DIRECTORY_SEPARATOR;
        }
        
        require_once(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'extension' . DIRECTORY_SEPARATOR . 'class' . DIRECTORY_SEPARATOR . 'dUnzip2.php');        
        $unzip = new dUnzip2($file);        
        $unzip->unzipAll($path);
        $unzip->close();
        unset($unzip);
        
        // show debug
        $this->showDebug('Unpack completed.');
    }
    
    /**
     * Execute init file if exists
     */
    private function executeInitFile()
    {
        // show debug
        $this->showDebug('Executing init file...');
        
        // verify if file exists
        $fileName = $this->tempDir . $this->extensionName . DIRECTORY_SEPARATOR . $this->extensionName . DIRECTORY_SEPARATOR . 'InitExtension.php';
        
        if (file_exists($fileName))
        {
            require_once($fileName);
            
            $InitExtension = new InitExtension(parent::getName(), parent::getCommandRunner());
            $InitExtension->run($this->args);                        
        }
                
        // show debug
        $this->showDebug('Init file completed.');
    }    
}